// Copyright 2015-2016 Dietrich Epp.
// This file is part of the Moria Gamedev Repository.  This file is licensed
// under the terms of the MIT license.  For more information, see LICENSE.txt.
'use strict';
var util = require('./util');

var MUSIC_FADE_TIME = 1;

var curAudio = null;
var curKey = null;
var nextKey = null;
var loaded = null;
var fading = false;

function update() {
	if (!nextKey || curAudio) {
		return;
	}
	if (!Game.cache.checkSoundKey(nextKey)) {
		return;
	}
	curAudio = Game.sound.add(nextKey, 1, true);
	curKey = nextKey;
	nextKey = null;
	curAudio.play();
}

function fileComplete(progress, key, success, loaded, total) {
	try {
	if (nextKey && key === nextKey) {
		var timer = Game.time.create(true);
		timer.add(0, update);
		timer.start();
	}
	} catch (e) {
		console.error(e);
	}
}

function fadeComplete() {
	curAudio.destroy();
	curAudio = null;
	curKey = null;
	fading = false;
	update();
}

function play(name) {
	if (util.hasVar('nomusic')) {
		return;
	}
	var key, paths;
	if (!name) {
		key = null;
	} else {
		key = 'music/' + name;
		paths = Assets.music[name];
		if (!paths) {
			console.error('No such music:', name);
			key = null;
		}
	}
	if (key === (nextKey || curKey)) {
		return;
	}
	if (curAudio && !fading) {
		curAudio.fadeOut(MUSIC_FADE_TIME * 1000);
		curAudio.onFadeComplete.addOnce(fadeComplete);
		fading = true;
	}
	nextKey = key;
	if (!key) {
		return;
	}
	if (!loaded) {
		loaded = {};
		Game.load.onFileComplete.add(fileComplete);
	}
	if (!loaded[key]) {
		var uris = [], i;
		for (i = 0; i < paths.length; i++) {
			uris.push('music/' + paths[i]);
		}
		Game.load.audio(key, uris);
		Game.load.start();
	}
	update();
}

module.exports = {
	play: play,
};
